package org.s4s.gui.views;

import com.vaadin.data.util.BeanContainer;
import com.vaadin.data.util.BeanItem;
import com.vaadin.event.ItemClickEvent;
import com.vaadin.navigator.ViewChangeListener;
import com.vaadin.server.FontAwesome;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Panel;
import com.vaadin.ui.Table;
import com.vaadin.ui.UI;
import com.vaadin.ui.VerticalLayout;
import java.sql.SQLException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.s4s.exceptions.DAOException;
import org.s4s.modell.dto.Product;
import org.s4s.modell.dto.Warenkorb;
import org.s4s.process.control.TransactionControl;
import org.s4s.services.util.Roles;
import org.s4s.services.util.Views;

/**
 *
 * @author Holger
 */
public class WarenkorbView extends TemplateView {

    private final Warenkorb warenkorb = Warenkorb.getInstance();
    private Product product = null;

    @Override
    public void enter(ViewChangeListener.ViewChangeEvent event) {
        if (session.getAttribute(Roles.CURRENTUSER) == null) {
            UI.getCurrent().getNavigator().navigateTo(Views.LOGIN);
            Notification.show(null, "Einloggen, um auf den Warenkorb zuzugreifen!", Notification.Type.WARNING_MESSAGE);
        }
        this.setUp();
    }

    public void setUp() {

        final VerticalLayout contentLayout = new VerticalLayout();
        super.setUpFooterAndHeader(contentLayout);

        if (warenkorb.getAnzahl() == 0) {
            Panel panel = new Panel("Der Warenkorb ist leer!");
            this.addComponent(panel);
            this.setComponentAlignment(panel, Alignment.TOP_LEFT);
        } else {
            HorizontalLayout horizontalLayout = new HorizontalLayout();
            HorizontalLayout buttonLayout = new HorizontalLayout();
            Panel pAnzahl = new Panel("In Ihrem Warenkorb sind momentan: " + warenkorb.getAnzahl() + " Artikel!");
            horizontalLayout.addComponent(pAnzahl);
            horizontalLayout.setComponentAlignment(pAnzahl, Alignment.TOP_LEFT);

            Panel pSum = new Panel("Gesamtwert im Warenkorb beträgt: " + warenkorb.sum());
            horizontalLayout.addComponent(pSum);
            horizontalLayout.setComponentAlignment(pSum, Alignment.TOP_RIGHT);

            this.addComponent(horizontalLayout);

            List<Product> list = warenkorb.inhalt();
            final BeanContainer<Integer, Product> data = new BeanContainer<>(Product.class);
            data.setBeanIdProperty("id");
            Table table = new Table("Ihr Warenkorb", data);
            table.setSizeFull();
            table.setSelectable(true);
            data.addAll(list);
            this.addComponent(table);
            table.setPageLength(table.size());
            table.setVisibleColumns(new Object[]{"name", "description", "price"});

            table.addItemClickListener(new ItemClickEvent.ItemClickListener() {
                @Override
                public void itemClick(ItemClickEvent event) {
                    BeanItem<Product> productBean = data.getItem(event.getItemId());
                    product = productBean.getBean();
                }
            });

            Button removeButton = new Button("Aus Warenkorb entfernen!");
            removeButton.addClickListener((Button.ClickEvent event) -> {
                if (this.product == null) {
                    Notification.show(null, "Bitte, das zu entferndende Produkt auswählen",
                            Notification.Type.HUMANIZED_MESSAGE);
                } else {
                    warenkorb.remove(product);
                    UI.getCurrent().getNavigator().navigateTo(Views.WARENKORB);
                }
            });

            Button buyButton = new Button("Kaufen!");
            buyButton.addClickListener((Button.ClickEvent event) -> {
                TransactionControl t = new TransactionControl();
                try {
                    t.createTransactions();
                    UI.getCurrent().getNavigator().navigateTo(Views.CHECKOUT);
                } catch (DAOException ex) {
                    Logger.getLogger(WarenkorbView.class.getName()).log(Level.SEVERE, null, ex);
                } catch (SQLException ex) {
                    Logger.getLogger(WarenkorbView.class.getName()).log(Level.SEVERE, null, ex);
                }
            });

            buttonLayout.addComponent(buyButton);
            buttonLayout.setComponentAlignment(buyButton, Alignment.BOTTOM_LEFT);
            buttonLayout.addComponent(removeButton);
            buttonLayout.setComponentAlignment(removeButton, Alignment.BOTTOM_RIGHT);
            this.addComponent(buttonLayout);
            this.setComponentAlignment(buttonLayout, Alignment.BOTTOM_CENTER);
            removeButton.setIcon(FontAwesome.UNDO);
        }

    }
}
